from __future__ import division

def print_info(router_obj):
    print('Details for router at %s:%d' % (router_obj.host, router_obj.port))
    print('')

    router_info = router_obj.get_router_info()
    print('Router %s %s running on firmware %s' % (router_obj.__class__.__name__, router_info.hardware_version, router_info.firmware_version))
    print('')

    uptime = router_obj.get_uptime()
    if uptime is None:
        print('Router uptime: UNKNOWN')
    else:
        print('Router uptime: %d seconds (~%d hours)' % (uptime, uptime / 3600))

    online_time = router_obj.get_pppoe_online_time()
    if online_time is None:
        print('Router online time: UNKNOWN')
    else:
        print('Router online time: %d seconds (~%d hours)' % (online_time, online_time / 3600))

    print('')

    stats = router_obj.get_traffic_stats()
    print('Traffic statistics:')
    print(' - Received %.2fMB (%d packets)' % (stats.bytes_recv / 1024 / 1024, stats.packets_recv))
    print(' - Sent %.2fMB (%d packets)' % (stats.bytes_sent / 1024 / 1024, stats.packets_sent))
    print('')


    print('Router MAC address: %s' % router_obj.get_mac_address())
    print('')

    print ('DNS servers: %s' % ', '.join(router_obj.get_dns_servers()))
    print('')


    connected_clients_list = router_obj.get_connected_clients_list()
    print('Connected: %d clients' % len(connected_clients_list))
    for item in connected_clients_list:
        name, mac, ip = item.client_name, item.mac, item.ip

        if item.lease_time is None:
            lease_info = ' [unknown lease time]'
        else:
            lease_info = '[PERMANENT]' if item.is_permanent_lease else ' - lease expires in %d seconds' % item.lease_time

        print(' - %s (%s / %s) %s' % (name, ip, mac, lease_info))
    print('')

    dmz_settings = router_obj.get_dmz_settings()
    if dmz_settings.is_supported:
        if dmz_settings.is_enabled:
            print('DMZ is active for %s' % dmz_settings.ip)
            dhcp_reservations = router_obj.get_addr_reservation_list()
            if not dhcp_reservations.has_ip(dmz_settings.ip):
                print(' - WARNING: no address reservation for the DMZ IP. This MAY be a problem')
            else:
                print(' - OK: %s is also in the address reservation list' % dmz_settings.ip)
        else:
            print('DMZ is disabled')
    else:
        print('DMZ is not supported')
    print('')


    dhcp_server = router_obj.get_dhcp_settings()
    if dhcp_server.is_enabled:
        print('DHCP server enabled with range %s - %s' % (dhcp_server.ip_start, dhcp_server.ip_end))
    else:
        print('DHCP server is disabled')


    reservation_list = router_obj.get_addr_reservation_list()
    if reservation_list.supports_reservations:
        print('DHCP address reservation list: %d addresses' % len(reservation_list))
        for item in reservation_list:
            mac, ip, is_enabled = item.mac, item.ip, item.is_enabled
            disabled_modifier = '' if is_enabled else '[DISABLED]'

            print(' - %s (%s) %s' % (ip, mac, disabled_modifier))
    else:
        print('Address reservations not supported')
    print('')


    wireless = router_obj.get_wireless_settings()
    if wireless.is_supported:
        print('Wireless settings:')
        if wireless.is_enabled:
            print(' - Wireless enabled')
        else:
            print(' - Wireless is supported, but DISABLED')

        ssid_modifier = '[public broadcast]' if wireless.is_broadcasting_ssid else '[PRIVATE broadcast]'
        print(' - SSID: %s %s' % (wireless.ssid, ssid_modifier))

        security_modifier = ''
        if wireless.security_type_is_wep and wireless.password is not None:
            security_modifier = '/HEX' if wireless.is_wep_password_in_hex else '/ASCII'
        print(' - Security: %s%s' % (wireless.security_type.upper(), security_modifier))

        if wireless.channel is None:
            print(' - Channel: <UNKNOWN>')
        else:
            print(' - Channel: %d' % wireless.channel)

        if wireless.password is None:
            print(' - Password: <UNKNOWN>')
        else:
            print(' - Password: %s' % wireless.password)
    else:
        print('Wireless is not supported!')
    print('')


def split_in_groups(item, group_size):
    """Splits an iterable in groups of tuples.

    If we take an incoming list/tuple like: ('id1', 'val1', 'id2', 'val2')
    and split it with group_size 2, we'll get: [('id1', 'val1'), ('id2', 'val2')]

    If we take a string like: 'abcdef'
    and split it with group_size 2, we'll get: ['ab', 'cd', 'ef']
    """
    return [item[idx_start:idx_start + group_size]
                for idx_start in range(0, len(item), group_size)]



class RouterInfo(object):
    """Represents some system information about the router,
    such as hardware and firmware version.
    """

    def __init__(self):
        self._hardware_version = None
        self._firmware_version = None

    def set_hardware_version(self, value):
        self._hardware_version = value

    @property
    def hardware_version(self):
        return self._hardware_version

    def set_firmware_version(self, value):
        self._firmware_version = value

    @property
    def firmware_version(self):
        return self._firmware_version



import re
import ast
import base64
import urllib

from roscraco.router.base import RouterBase

from roscraco.helper import validator, converter, split_in_groups
from roscraco.exception import RouterParseError, RouterIdentityError, \
     RouterError

from roscraco.response import RouterInfo, TrafficStats, DMZSettings, \
     ConnectedClientsListItem, ConnectedClientsList, \
     DHCPReservationList, DHCPReservationListItem, DHCPServerSettings


class TplinkBase(RouterBase):

    def _perform_http_request(self, *args, **kwargs):
        # Without the Accept-Encoding header the web-interface of the
        # WR941N router (and potentially some other models) crashes.
        # A bug report was submitted to TpLink, but still no progress.
        # Even if they fix it, thousands of routers out there would still
        # run the old firmware and break.
        auth = base64.b64encode('%s:%s' % (self.username, self.password))
        kwargs['headers'] = [
            ('Accept-Encoding', 'gzip,deflate'),
            ('Authorization', 'Basic %s' % auth),
        ]
        return RouterBase._perform_http_request(self, *args, **kwargs)

    def _get_status_array(self, array_name):
        contents = self._make_http_request_read('StatusRpm.htm')
        return _extract_js_array_data(contents, array_name)

    def get_router_info(self):
        return _parse_router_info(self._get_status_array('statusPara'))

    def get_mac_address(self):
        return _parse_mac_address(self._get_status_array('wanPara'))

    def get_dns_servers(self):
        return _parse_dns_servers(self._get_status_array('wanPara'))

    def get_pppoe_online_time(self):
        return _parse_pppoe_online_time(self._get_status_array('wanPara'))

    def get_uptime(self):
        return _parse_uptime(self._get_status_array('statusPara'))

    def get_traffic_stats(self):
        return _parse_traffic_stats(self._get_status_array('statistList'))

    def get_connected_clients_list(self):
        return _parse_connected_clients_list(self._make_http_request_read('AssignedIpAddrListRpm.htm'))

    def get_dmz_settings(self):
        return _parse_dmz_settings(self._make_http_request_read('DMZRpm.htm'))

    def push_dmz_settings(self, settings):
        get_params = _generate_dmz_data(settings)
        contents = self._make_http_request_write('DMZRpm.htm?%s' % urllib.urlencode(get_params))
        return _parse_dmz_settings(contents) == settings

    def get_addr_reservation_list(self):
        return _parse_addr_reservation_list(self._make_http_request_read('FixMapCfgRpm.htm'))

    def push_addr_reservation_list(self, lst_new):
        lst_new.ensure_valid()

        lst_old = self.get_addr_reservation_list()
        if lst_old == lst_new:
            return True

        # delete all old (currently added) items first
        for _ in lst_old:
            # delete from the front (id=0) as many times as needed
            self._make_http_request_write('FixMapCfgRpm.htm?Del=0')

        for item in lst_new:
            data = _generate_addr_reservation_item_data(item)
            contents = self._make_http_request_write('FixMapCfgRpm.htm?%s' % urllib.urlencode(data))

            if 'var errCode = "' in contents:
                # could be several things (1. already added; 2. invalid MAC (broadcast..); ..)
                raise RouterError('Error while pushing %s/%s' % (item.ip, item.mac))

        return self.get_addr_reservation_list() == lst_new

    def get_dhcp_settings(self):
        return _parse_dhcp_settings(self._make_http_request_read('LanDhcpServerRpm.htm'))

    @property
    def supports_reboot(self):
        return True

    def reboot(self):
        self._make_http_request_write('SysRebootRpm.htm?Reboot=Reboot')

    @property
    def url_base(self):
        return 'http://%s:%d/userRpm/' % (self.host, self.port)

    def _ensure_www_auth_header(self, header_value_expected):
        info = self._perform_http_request('%sStatusRpm.htm' % self.url_base)[1]
        header_auth = info.getheader('WWW-Authenticate')
        if header_auth != header_value_expected:
            raise RouterIdentityError(
                'Bad or missing WWW-Authenticate header: %s/%s' %
                    (header_auth, header_value_expected)
            )

def is_valid_mac_address(mac):
    """Validates a MAC address in the format that TPLink routers expects."""
    regex = re.compile('(([a-fA-F0-9]{2}-){5})([a-fA-F0-9]{2})$')
    return regex.match(mac) is not None


def _extract_js_array_data(contents, array_name):
    """Extracts the contents of a javascript array on the page.

    The TP-Link control panel often uses javascript arrays to store some
    of the data on the page that would later be used to generate the UI.

    Here's an example of an array full of data:

    var statistList = new Array(
    14131821, 1757256, 16065, 12432,
    0,0 );

    What we're actually doing below is:
    1) Find where the real array data starts (after the `(` char)
    2) Find where the real array data ends (after `);`)
    3) Safely parse the array data `(..)` as a tuple
    """
    try:
        find = 'var %s = new Array(' % array_name
        start = contents.index(find) + find.__len__()
        end = contents.index(');', start)
        array_contents = '(%s)' % contents[start:end]

        result = ast.literal_eval(array_contents)
        if not isinstance(result, tuple):
            raise RouterParseError('Bad javascript array evaluation. Result not a tuple!')
        # ast.literal_eval may mess up our nice unicode strings
        # depending on the default system encoding (usually ascii)
        result = tuple([v.decode('utf-8', 'ignore')
                        if isinstance(v, bytes) else v for v in result])
        return result
    except Exception, e:
        raise RouterParseError('Failed at evaluating array %s: %s' % (array_name, repr(e)))


def _parse_pppoe_online_time(data_array):
    try:
        uptime_string = data_array[12] # `0 day(s) 10:11:12`

        # non-pppoe routers have a different array format.. no uptime there
        if uptime_string == '':
            return None

        return _parse_uptime_to_seconds(uptime_string)
    except IndexError, e:
        raise RouterParseError('Cannot access the array index: %s' % repr(e))


def _parse_uptime_to_seconds(string):
    """Parses an uptime string such as `0 day(s) 10:11:12`
    or `0 days 10:18:45` to seconds.

    These 2 strings both appear in the router interface.
    """
    regex = re.compile('([0-9]+) (?:days|day\(s\)) ([0-9]+):([0-9]+):([0-9]+)')
    match_object = regex.match(string)
    if match_object is None:
        raise RouterParseError('Invalid uptime string `%s`' % str(string))

    days, hours, minutes, seconds = map(int, match_object.groups())
    return days * 86400 + hours * 3600 + minutes * 60 + seconds


def _parse_uptime(data_array):
    try:
        return int(data_array[4])
    except IndexError, e:
        raise RouterParseError('Cannot access the array index: %s' % repr(e))


def _parse_router_info(data_array):
    try:
        obj = RouterInfo()
        obj.set_hardware_version(data_array[6])
        obj.set_firmware_version(data_array[5])

        return obj
    except IndexError, e:
        raise RouterParseError('Cannot access the array index: %s' % repr(e))


def _parse_mac_address(data_array):
    try:
        return converter.normalize_mac(data_array[1])
    except IndexError, e:
        raise RouterParseError('Cannot access the array index: %s' % repr(e))


def _parse_dns_servers(data_array):
    try:
        dns_ips = data_array[11].split(' , ')
        return [ip.strip(' ') for ip in dns_ips if validator.is_valid_ip_address(ip)]
    except IndexError, e:
        raise RouterParseError('Cannot access the array index: %s' % repr(e))


def _parse_traffic_stats(data_array):
    data_array = data_array[:4]
    if len(data_array) != 4:
        raise RouterParseError('Unexpected stats size: %d' % len(data_array))

    data_array = map(int, list(data_array))
    return TrafficStats(*data_array)


def _parse_lease_time(string):
    """Parses a lease time string such as `04:23:15` to seconds.

    The format is HH:MM:SS
    """
    parts = string.split(':')
    if len(parts) != 3:
        raise RouterParseError('Cannot parse lease time string: %s' % string)

    try:
        parts = map(int, parts)
    except ValueError, e:
        raise RouterParseError('Found non-numeric part in lease time %s: %s' % (string, repr(e)))

    hours, minutes, seconds = parts

    return hours * 3600 + minutes * 60 + seconds


def _parse_dmz_settings(contents):
    array_name = 'DMZInf'

    result = _extract_js_array_data(contents, array_name)

    try:
        ip = result[1].strip(' ')
        if not validator.is_valid_ip_address(ip):
            raise RouterParseError('Invalid IP address: %s' % ip)

        obj = DMZSettings()
        obj.set_supported_status(True)
        obj.set_enabled_status(result[0] == 1)
        obj.set_ip(result[1])

        return obj
    except IndexError, e:
        raise RouterParseError(repr(e))


def _parse_addr_reservation_list(contents):
    array_name = 'dhcpList'

    result = _extract_js_array_data(contents, array_name)
    result = result[:-2]    # the last 2 elements are not needed

    # each 3 subsequent items are related (mac_address, ip, is_enabled)
    list_raw = split_in_groups(result, 3)

    reservation_list = DHCPReservationList()
    for mac, ip, is_enabled in list_raw:
        item = DHCPReservationListItem()
        item.set_mac(converter.normalize_mac(mac))
        item.set_ip(ip)
        item.set_enabled_status(is_enabled == 1)
        reservation_list.append(item)

    return reservation_list


def _parse_connected_clients_list(html):
    # the last 2 elements of the data array are not needed
    result = _extract_js_array_data(html, 'DHCPDynList')[:-2]

    lst = ConnectedClientsList()

    # each 4 subsequent items are related (client_name, mac_address, ip, lease_time)
    for client_name, mac, ip, lease_time in split_in_groups(result, 4):
        if not validator.is_valid_ip_address(ip):
            raise RouterParseError('Invalid IP address: %s' % ip)

        item = ConnectedClientsListItem()
        item.set_client_name(client_name)
        item.set_mac(converter.normalize_mac(mac))
        item.set_ip(ip)

        if lease_time == 'Permanent':
            item.set_lease_time(item.__class__.LEASE_TIME_PERMANENT)
        else:
            item.set_lease_time(_parse_lease_time(lease_time))

        lst.append(item)

    return lst


def _parse_dhcp_settings(html):
    settings = DHCPServerSettings()

    array = _extract_js_array_data(html, 'DHCPPara')

    try:
        settings.set_enabled_status(int(array[0]) == 1)
        settings.set_ip_start(array[1])
        settings.set_ip_end(array[2])
    except IndexError, e:
        raise RouterParseError(repr(e))

    settings.ensure_valid()
    return settings


def _generate_dmz_data(settings):
    settings.ensure_valid()

    get_params = {}
    get_params['ipAddr'] = settings.ip
    get_params['enable'] = 1 if settings.is_enabled else 0
    get_params['netMask'] = '255.255.255.0'
    get_params['Save'] = 'Save'
    return get_params


def _denormalize_mac(mac):
    """Takes a normalized mac address (all lowercase hex, no separators)
    and converts it to the TpLink format.

    Example::
        _denormalize_mac('abcdef123456') == 'ab-cd-ef-12-34-56'
    """
    return '-'.join((mac[i] + mac[i+1] for i in range(0, 12, 2)))


def _generate_addr_reservation_item_data(item):
    data = {}
    data['Mac'] = _denormalize_mac(item.mac)
    data['Ip'] = item.ip
    data['State'] = 1 if item.is_enabled else 0
    data['Changed'] = 0
    data['SelIndex'] = 0
    data['Page'] = 1
    data['Save'] = 'Save'
    return data



from ..exception import RouterParseError

def ip2long(ip_addr):
    """Converts an IP address string to an integer."""
    from socket import inet_aton
    from struct import unpack
    ip_packed = inet_aton(ip_addr)
    ip = unpack('!L', ip_packed)[0]
    return ip


def long2ip(ip):
    """Converts an integer representation of an IP address to string."""
    from socket import inet_ntoa
    from struct import pack
    return inet_ntoa(pack('!L', ip))


def normalize_mac(mac):
    """Converts any MAC address format to lowercase HEX with no separators."""
    from validator import is_valid_mac_address
    mac = mac.lower()
    if not is_valid_mac_address(mac):
        raise RouterParseError('MAC address %s is invalid!' % mac)
    return mac.replace(':', '').replace('-', '')



class ConnectedClientsList(list):
    """A response class representing the clients connected
    to the router at this time (or that have been recently connected).
    """
    pass


class ConnectedClientsListItem(object):
    """A client entry in the :class:`ConnectedClientsList`."""

    LEASE_TIME_PERMANENT = 'Permanent'

    def __init__(self):
        self._client_name = None
        self._mac_address = None
        self._ip_address = None
        self._lease_time = None

    def set_client_name(self, value):
        self._client_name = value

    @property
    def client_name(self):
        return self._client_name

    def set_mac(self, value):
        self._mac_address = value

    @property
    def mac(self):
        return self._mac_address

    def set_ip(self, value):
        self._ip_address = value

    @property
    def ip(self):
        return self._ip_address

    def set_lease_time(self, value):
        self._lease_time = value

    @property
    def lease_time(self):
        return self._lease_time

    @property
    def is_permanent_lease(self):
        return (self._lease_time == self.__class__.LEASE_TIME_PERMANENT)

    def __repr__(self):
        return '<%s: %s; %s>' % (self.__class__, self._client_name, self._ip_address)



import re
from ..exception import RouterError


WEP_CONSTRAINTS_MAP = {
    64: {'ascii': 5, 'hex': 10},
    128: {'ascii': 13, 'hex': 26},
    152: {'ascii': 16, 'hex': 32}
}


def is_valid_mac_address(mac):
    if is_valid_mac_address_normalized(mac):
        return True

    # The following regular expressions could probably
    # be merged into a single more complex one

    # separated by `-`
    regex = re.compile('(([a-fA-F0-9]{2}-){5})([a-fA-F0-9]{2})$')
    if regex.match(mac) is not None:
        return True

    # separated by `:`
    regex = re.compile('(([a-fA-F0-9]{2}:){5})([a-fA-F0-9]{2})$')
    if regex.match(mac) is not None:
        return True

    return False


def is_valid_mac_address_normalized(mac):
    """Validates that the given MAC address has
    what we call a normalized format.

    We've accepted the HEX only format (lowercase, no separators) to be generic.
    """
    return re.compile('^([a-f0-9]){12}$').match(mac) is not None


def is_valid_ip_address(ip):
    parts = ip.split('.')
    if len(parts) != 4:
        return False

    try:
        parts = map(int, parts)
    except ValueError:
        return False

    for part in parts:
        if part < 0 or part > 255:
            return False

    return True


def _is_hex_string(string):
    return re.compile('^([a-fA-F0-9]+)$').match(string) is not None


def is_valid_wpa_psk_password(password):
    try:
        password = password.decode('ascii')
    except (UnicodeDecodeError, UnicodeEncodeError):
        return False
    else:
        if password.strip(' ') != password:
            return False

        return 8 <= len(password) <= 63

def is_valid_wep_password(password, bit_length):
    """Validates a WEP password of the specified bit length,
    which imposes certain constraints on what's allowed."""


    # the password could be either HEX or ASCII
    # HEX is valid ASCII too
    try:
        password = password.decode('ascii')
    except (UnicodeDecodeError, UnicodeEncodeError):
        return False

    if password.strip(' ') != password:
        return False

    length = len(password)

    try:
        constraints = WEP_CONSTRAINTS_MAP[bit_length]

        if length == constraints['ascii']:
            # any ascii character works
            return True
        elif length == constraints['hex']:
            # if the string is that long, it can only be in HEX
            return _is_hex_string(password)

        return False
    except KeyError:
        raise RouterError('Invalid bit length: %d' % int(bit_length))


def is_wep_password_in_hex(password, bit_length):
    """Tells whether we're using HEX or ASCII for the specified password."""
    if password is None:
        return False
    # the password could be either HEX or ASCII
    # HEX is valid ASCII too
    try:
        password = password.decode('ascii')
    except (UnicodeDecodeError, UnicodeEncodeError):
        raise RouterError('Cannot validate password as HEX or ASCII: %s' %
                          password)

    length = len(password)

    try:
        constraints = WEP_CONSTRAINTS_MAP[bit_length]

        if length == constraints['ascii']:
            return False
        elif length == constraints['hex']:
            # if the string is that long, it can only be in HEX
            return _is_hex_string(password)

        return False
    except KeyError:
        raise RouterError('Invalid bit length: %d' % int(bit_length))


def is_valid_ssid(ssid):
    try:
        ssid = ssid.decode('ascii')
    except (UnicodeDecodeError, UnicodeEncodeError):
        return False
    else:
        if ssid.strip(' ') != ssid:
            return False
        return 2 <= len(ssid) <= 32


def is_ip_in_range(ip, ip_range_start, ip_range_end):
    """Tells whether the given IP (in string format) is inside the range."""

    from .converter import ip2long

    return ip2long(ip_range_start) <= ip2long(ip) <= ip2long(ip_range_end)



import base64
import re

from roscraco.router.base import RouterBase
from roscraco.helper import validator, converter, split_in_groups
from roscraco.exception import RouterParseError, RouterIdentityError

from roscraco.response import RouterInfo, TrafficStats, DMZSettings, \
     ConnectedClientsListItem, ConnectedClientsList, WirelessSettings, \
     DHCPReservationList, DHCPReservationListItem, DHCPServerSettings


class Tenda_W268R(RouterBase):

    def _perform_http_request(self, *args, **kwargs):
        auth = base64.b64encode('%s:%s' % (self.username, self.password))
        kwargs['headers'] = [
            ('Authorization', 'Basic %s' % auth),
        ]
        return RouterBase._perform_http_request(self, *args, **kwargs)

    def confirm_identity(self):
        uri = '%stop.html' % self.url_base
        _, headers, contents = self._perform_http_request(uri)

        header_server = headers.getheader('Server')
        if header_server != 'GoAhead-Webs':
            raise RouterIdentityError('Invalid Server header: %s' % header_server)

        if 'tenda_logo.jpg' not in contents:
            raise RouterIdentityError('Cannot confirm that this is a Tenda device.')

    def get_router_info(self):
        html = self._make_http_request_read('system_status.asp')
        return _parse_router_info(html)

    def get_uptime(self):
        html = self._make_http_request_read('system_status.asp')
        return _parse_uptime(html)

    def get_pppoe_online_time(self):
        html = self._make_http_request_read('system_status.asp')
        return _parse_pppoe_online_time(html)

    def get_traffic_stats(self):
        # This router doesn't report traffic statistics
        return TrafficStats(0, 0, 0, 0)

    def get_mac_address(self):
        html = self._make_http_request_read('system_status.asp')
        return _parse_mac_address(html)

    def get_dns_servers(self):
        html = self._make_http_request_read('system_status.asp')
        return _parse_dns_servers(html)

    def get_connected_clients_list(self):
        html = self._make_http_request_read('lan_dhcp_clients.asp')
        return _parse_connected_clients_list(html)

    def get_dmz_settings(self):
        html = self._make_http_request_read('nat_dmz.asp')
        return _parse_dmz_settings(html)

    def push_dmz_settings(self, settings):
        post_params = _generate_dmz_data(settings)
        contents = self._make_http_request_write('goform/VirSerDMZ', post_params)
        return _parse_dmz_settings(contents) == settings

    def get_dhcp_settings(self):
        html = self._make_http_request_read('lan_dhcps.asp')
        return _parse_dhcp_settings(html)

    def get_addr_reservation_list(self):
        html = self._make_http_request_read('lan_dhcp_clients.asp')
        return _parse_addr_reservation_list(html)

    def push_addr_reservation_list(self, lst_new):
        post_params = _generate_addr_reservation_data(lst_new)
        contents = self._make_http_request_write('goform/DhcpListClient', post_params)
        return _parse_addr_reservation_list(contents) == lst_new

    def get_wireless_settings(self):
        basic = self._make_http_request_read('wireless_basic.asp')
        security_settings = self._make_http_request_read('goform/wirelessGetSecurity')
        security = self._make_http_request_read('wireless_security.asp')
        return _parse_wireless_settings(basic, security, security_settings)

    def push_wireless_settings(self, settings):
        settings_old = self.get_wireless_settings()

        post_params = _generate_wireless_data_basic(settings)
        basic = self._make_http_request_write('goform/wirelessBasic', post_params)

        post_params = _generate_wireless_data_security(settings)
        security = self._make_http_request_write('goform/APSecurity', post_params)

        # Let's validate that all changes were saved as requested..

        if settings_old.is_enabled and not settings.is_enabled:
            # Going from enabled to disabled prevents some of the basic
            # settings from being fully commited.
            # Let's skip validation in that case and hope for the best.
            return True

        security_settings = self._make_http_request_read('goform/wirelessGetSecurity')
        settings_new = _parse_wireless_settings(basic, security, security_settings)
        return settings_new == settings

    @property
    def supports_reboot(self):
        return True

    def reboot(self):
        self._make_http_request_write('goform/SysToolReboot')


def _parse_router_info(html):
    obj = RouterInfo()

    match_obj = re.compile('hw_ver="(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine hardware version')
    obj.set_hardware_version(match_obj.group(1))

    match_obj = re.compile('run_code_ver="(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine firmware version')
    obj.set_firmware_version(match_obj.group(1))
    return obj


def _parse_uptime(html):
    match_obj = re.compile('uptime=\s"(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine uptime')
    return int(match_obj.group(1))


def _parse_pppoe_online_time(html):
    match_obj = re.compile('conntime="(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine connection time')
    return int(match_obj.group(1))


def _parse_mac_address(html):
    match_obj = re.compile('wan_mac="(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine WAN mac address')
    return converter.normalize_mac(match_obj.group(1))


def _parse_dns_servers(html):
    dns_ips = re.compile('dns[12]="(.+?)";').findall(html)
    return [ip.strip(' ') for ip in dns_ips if validator.is_valid_ip_address(ip)]


def _parse_connected_clients_list(html):
    dhcp_list = re.compile('var dhcpList=new Array\((.*)\);').search(html)
    if dhcp_list is None:
        raise RouterParseError('Cannot find DHCP list.')
    dhcp_list = dhcp_list.group(1)

    results = re.compile("'(.+?);(.+?);(.+?);[01];(\d+)'").findall(dhcp_list)
    lst = ConnectedClientsList()
    for client_name, ip, mac, lease_time in results:
        if not validator.is_valid_ip_address(ip):
            raise RouterParseError('Invalid IP address: %s' % ip)
        if not validator.is_valid_mac_address(mac):
            raise RouterParseError('Invalid MAC address: %s' % mac)

        item = ConnectedClientsListItem()
        item.set_client_name(client_name)
        item.set_mac(converter.normalize_mac(mac))
        item.set_ip(ip)
        item.set_lease_time(int(lease_time))

        lst.append(item)

    return lst


def _parse_dmz_settings(html):
    settings = DMZSettings()
    settings.set_supported_status(True)
    settings.set_reboot_requirement_status(False)

    match_obj = re.compile('var\sdef_DMZIP\s=\s"(.*)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine DMZ IP address')
    settings.set_ip(match_obj.group(1))

    match_obj = re.compile('var\sdef_dmzen\s=\s"(.+?)";').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine DMZ enabled status')
    settings.set_enabled_status(match_obj.group(1) == '1')
    return settings


def _parse_dhcp_settings(html):
    settings = DHCPServerSettings()

    match_obj = re.compile('document.LANDhcpsSet.DHEN.checked = (0|1);').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine DHCP enabled status')
    settings.set_enabled_status(match_obj.group(1) == '1')

    match_obj = re.compile('document.LANDhcpsSet.dips.value = \(\("(.+?)"').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine DHCP start IP address')
    ip_start = match_obj.group(1)
    if not validator.is_valid_ip_address(ip_start):
        raise RouterParseError('IP start address is invalid: %s' % ip_start)
    settings.set_ip_start(ip_start)

    match_obj = re.compile('document.LANDhcpsSet.dipe.value = \(\("(.+?)"').search(html)
    if match_obj is None:
        raise RouterParseError('Cannot determine DHCP end IP address')
    ip_end = match_obj.group(1)
    if not validator.is_valid_ip_address(ip_end):
        raise RouterParseError('IP end address is invalid: %s' % ip_end)
    settings.set_ip_end(ip_end)

    return settings


def _parse_addr_reservation_list(html):
    reservations = re.compile('var StaticList = new Array\((.*)\);').search(html)
    if reservations is None:
        raise RouterParseError('Cannot find reservations list.')
    reservations_list = reservations.group(1)

    results = re.compile(';(.+?);(.+?);([12]);(\d+)').findall(reservations_list)
    reservation_list = DHCPReservationList()
    reservation_list.set_reboot_requirement_status(False)
    for ip, mac, mac_bind, _time in results:
        item = DHCPReservationListItem()
        item.set_mac(converter.normalize_mac(mac))
        item.set_ip(ip)
        item.set_enabled_status(mac_bind == '1')
        reservation_list.append(item)
    return reservation_list


def _parse_wireless_settings(basic, security, security_settings):
    settings = WirelessSettings()
    settings.add_security_support(WirelessSettings.SECURITY_TYPE_WEP64)
    settings.add_security_support(WirelessSettings.SECURITY_TYPE_WEP128)
    settings.add_security_support(WirelessSettings.SECURITY_TYPE_WPA)
    settings.add_security_support(WirelessSettings.SECURITY_TYPE_WPA2)
    # Changes take effect immediately without needing to reboot.
    settings.set_reboot_requirement_status(False)

    match_obj = re.compile('var enablewireless= "([01])";').search(basic)
    if match_obj is None:
        raise RouterParseError('Cannot determine wireless enabled status.')
    settings.set_enabled_status(match_obj.group(1) == '1')

    match_obj = re.compile('var channel_index  = "(\d+)";').search(basic)
    if match_obj is None:
        raise RouterParseError('Cannot determine wireless channel.')
    settings.set_channel(int(match_obj.group(1)))

    match_obj = re.compile('var broadcastssidEnable  = "([01])";').search(basic)
    if match_obj is None:
        raise RouterParseError('Cannot determine broadcast status.')
    # this is reversed, 0 means enable..
    settings.set_ssid_broadcast_status(match_obj.group(1) == '0')

    security_params = security_settings.strip().split('\r')
    settings.set_ssid(security_params[0])

    sec_type = security_params[2]
    if sec_type == 'WPA2PSK':
        security_type = WirelessSettings.SECURITY_TYPE_WPA2
        settings.set_password(security_params[13])
    elif sec_type == 'WPAPSK':
        security_type = WirelessSettings.SECURITY_TYPE_WPA
        settings.set_password(security_params[13])
    elif sec_type == 'WEPAUTO':
        # let's determine wep64 or wep128 by inspecting the first password
        wep_key_1 = security_params[6]
        if len(wep_key_1) in (5, 10):
            # 5 chars is for ASCII passwords, 10 is for HEX
            security_type = WirelessSettings.SECURITY_TYPE_WEP64
        else:
            security_type = WirelessSettings.SECURITY_TYPE_WEP128
        settings.set_password(wep_key_1)
    else:
        security_type = WirelessSettings.SECURITY_TYPE_NONE
    settings.set_security_type(security_type)

    return settings


def _generate_dmz_data(settings):
    settings.ensure_valid()
    post_params = {}
    post_params['GO'] = 'nat_dmz.asp'
    post_params['en'] = 1 if settings.is_enabled else 0
    post_params['dmzip'] = settings.ip
    return post_params


def _generate_addr_reservation_data(lst):
    lst.ensure_valid()

    def denormalize_mac(mac):
        # abcdef123456 => AB:CD:EF:12:34:45
        byte_octets = split_in_groups(mac, 2)
        return (':'.join(byte_octets)).upper()

    post_params = {}
    post_params['LISTLEN'] = len(lst)

    # a sequence of 1 or 0 flags, saying
    # whether the given item is enabled
    ip_mac_bind = ''

    for i, item in enumerate(lst):
        item_data = ';%s;%s;%d;86400' % (
            item.ip,
            denormalize_mac(item.mac),
            1 if item.is_enabled else 2,
        )
        post_params['list%d' % (i + 1)] = item_data
        ip_mac_bind += '1' if item.is_enabled else '0'

    post_params['IpMacEN'] = ip_mac_bind

    return post_params


def _generate_wireless_data_basic(settings):
    settings.ensure_valid()

    post_params = {}
    post_params['enablewirelessEx'] = 1 if settings.is_enabled else 0
    if settings.is_enabled:
        post_params['enablewireless'] = 1

    post_params['ssid'] = settings.ssid
    post_params['broadcastssid'] = int(settings.is_broadcasting_ssid)
    post_params['sz11gChannel'] = settings.channel

    # Below are some settings that we force on everyone,
    # because we don't handle them.
    post_params['bssid_num'] = 1 # hardcoded by default too, unknown purpose
    post_params['n_mode'] = 0 # Operating Mode (0: Mixed, 1: Green)
    post_params['wirelessmode'] = 9 # b/g/n
    post_params['n_bandwidth'] = 1 # Channel bandwidth (0: 20, 1: 20/40)
    post_params['n_gi'] = 1 # Guard interval = auto
    post_params['n_mcs'] = 33 # MCS = auto
    post_params['n_rdg'] = 1 # Reverse Direction Grant = enabled
    post_params['n_extcha'] = 0 # Extension channel
    post_params['n_amsdu'] = 0 # Aggregation MSDU = disabled

    return post_params


def _generate_wireless_data_security(settings):
    settings.ensure_valid()

    post_params = {}
    post_params['ssidIndex'] = 0 # unknown field purpose

    security_mode_map = {
        settings.SECURITY_TYPE_WPA2: 'WPA2PSK',
        settings.SECURITY_TYPE_WPA: 'WPAPSK',
        settings.SECURITY_TYPE_WEP64: 'WEPAUTO',
        settings.SECURITY_TYPE_WEP128: 'WEPAUTO',
        settings.SECURITY_TYPE_NONE: 'Disable',
    }
    security_mode = security_mode_map.get(settings.security_type, settings.SECURITY_TYPE_NONE)
    post_params['security_mode'] = security_mode
    post_params['security_shared_mode'] = 'WEP'

    post_params['wep_default_key'] = 1
    for i in range(1, 5):
        post_params['wep_key_%d' % i] = ''
        post_params['WEP%dSelect' % i] = 0

    if settings.security_type_is_wep:
        post_params['wep_key_1'] = settings.password
        post_params['WEP1Select'] = 0 if settings.is_wep_password_in_hex else 1
    elif settings.security_type_is_wpa:
        post_params['passphrase'] = settings.password
        post_params['keyRenewalInterval'] = 3600
        post_params['cipher'] = 1 # 1: AES, 0: TKIP, 2: AES & TKIP

    return post_params



__doc__ = open('README.rst').read()
__doc__ += """

Links
-----

* `source <https://github.com/spantaleev/roscraco>`_
"""

from setuptools import setup, find_packages

import roscraco


setup(
    name = "roscraco",
    version = roscraco.__version__,
    description = "A library for managing home routers (networking equipment).",
    long_description = __doc__,
    author = "Slavi Pantaleev",
    author_email = "s.pantaleev@gmail.com",
    url = "https://github.com/spantaleev/roscraco",
    keywords = ["router", "networking", "tplink",
                "canyon", "netgear", "zyxel", "tomato"],
    platforms = "any",
    license = "BSD",
    packages = find_packages(),
    install_requires = [],
    classifiers = [
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Telecommunications Industry",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: System :: Networking",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)




from wr340g import Tplink_WR340G, Tplink_WR340Gv4

from wr720n import Tplink_WR720N

from wr740n import Tplink_WR740N
from wr741n import Tplink_WR741N

from wr940n import Tplink_WR940N
from wr941n import Tplink_WR941N
